# -*- coding: utf8 -*-
'''
querytool preferences parsing
###############################
$RCSfile: preferences.py,v $
$Revision: 457 $
$Author: markus $
$Date: 2005-11-03 15:23:37 +0100 (Thu, 03 Nov 2005) $
'''


import os.path
import xml.sax, xml.sax.handler
from biocase.wrapper.errorclasses         import *
from biocase.tools.various_functions     import isTrue



# ------------------------------------------------------------------------------------
class QTPreferences:
    '''Querytool preferences class. Stores datasource specific preferences.'''
    def __init__(self):
        self.skin = 'default'
        self.defaultSchema = None # first schema label of the QT prefs file
        self.schemas  = {} # key=label, value=QTSchemaPreferences objects

    def __repr__(self):
        return '''QTPrefs: skin=%s, def.schema=%s, schemas=%s'''%(self.skin, self.defaultSchema, str(self.schemas.values()))


# ------------------------------------------------------------------------------------
class QTSchemaPreferences:
    '''Querytool preferences for a single schema.'''
    def __init__(self):
        self.label = None # label of the schema
        self.NS    = None # the namespace
        self.concepts = {} # key=label, value=QTConceptPreferences object
        self.form     = [] # list of concept labels that are shown on the main form
        self.grouping = None # the default concept used for grouping
        self.reclistXSL = None # the stylesheet to be used for the record list page
        self.limit = 100 # the record limit used for the recordlist paging
        self.details  = {} # key=detail name, value=detail object
     
    def __repr__(self):
        return '''\n  QTSchema: label=%s, ns=%s, grouping=%s, reclist=%s, details=%s, form=%s, concepts=%s\n'''%(self.label, self.NS, self.grouping, self.reclistXSL, str(self.details.values()), str(self.form), str(self.concepts.values()) )

    def knowsLabel(self, label):
        if self.getConceptFromLabel is not None:
            return True
        else:
            return False

    def getConceptFromLabel(self, label):
        # try to find label in list of concepts first
        if self.concepts.has_key(label):
            return self.concepts[label]
        # try to find label in rec IDs of detail pages
        for detailObj in self.details.values():
            if detailObj.recID.has_key(label):
                recID  = detailObj.recID[label]
                return QTConceptPreferences(label=label, path=recID.path, cops='=')
        return None
     
# ------------------------------------------------------------------------------------
class QTConceptPreferences:
    '''Querytool preferences for a single concept.'''
    def __init__(self, label=None, path=None, cops='~'):
        self.label = label
        self.path  = path
        self.cops  = cops  # the allowed comparison operators

    def __repr__(self):
        return '''\n    QTConcept: label=%s, path=%s, cops=%s'''%(self.label, self.path, self.cops)

        
# ------------------------------------------------------------------------------------
class QTDetailPreferences:
    '''Querytool preferences for a single schema.'''
    def __init__(self):
        self.name = None # name of the detail
        self.stylesheet = None # the default stylesheet
        self.recID = {} # dict of QTRecID objects, key=parameter name, val=Obj
        self.notAvailableMessage = 'There was no result from the wrapper.'

    def __repr__(self):
        return '''QTDetail: name=%s, xsl=%s, recIDs=%s'''%(self.name, self.stylesheet, str(self.recID))


# ------------------------------------------------------------------------------------
class QTRecID:
    '''Record identifier object.'''
    def __init__(self, para=None, path=None):
        self.parameter = para # name of the GET parameter
        self.path = path # concept xpath

    def __repr__(self):
        return '''QTRecID: parameter=%s, path=%s'''%(self.parameter, self.path)


# ------------------------------------------------------------------------------------
class QTPreferencesHandler(xml.sax.handler.ContentHandler):
    '''parses the provider setup configuration file 
    and fills a psf object with its data.'''
    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def __init__(self, QTPrefObj):
        self.QTPrefObj = QTPrefObj
        # list of tablealias information used to init the graph
        self._isSchema = 0
        self._value = ''
        xml.sax.handler.ContentHandler.__init__(self)
    
    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def startElement(self, name, attrs):
        self._value = ''
        if name == 'Skin':
            self.QTPrefObj.skin = attrs.get('name', 'default')
        elif name == 'Schema':
            self._isSchema   = 1
            self._schema = QTSchemaPreferences()
            self._schema.NS    = attrs.get('ns', None)
            self._schema.label = attrs.get('label', None)
            if self.QTPrefObj.defaultSchema is None:
                # the first schema is the default one:
                self.QTPrefObj.defaultSchema = self._schema.label
        if self._isSchema:
            if name == 'Concept':
                if attrs.get('label', None) is not None and len(attrs.get('label', '')) > 0:
                    conObj = QTConceptPreferences()
                    conObj.label = attrs['label']
                    conObj.path  = attrs.get('path', None)
                    conObj.cops  = attrs.get('cops', '~')
                    self._schema.concepts[conObj.label] = conObj
                    if isTrue( attrs.get('searchform', 'false') ):
                        self._schema.form.append( attrs['label'] )
            elif name == 'Grouping':
                self._schema.grouping = attrs.get('concept', None)
            elif name == 'Recordlist':
                self._schema.reclistXSL = attrs.get('stylesheet', None)
                try:
                    self._schema.limit = int( attrs.get('limit', 100) )
                except:
                    self._schema.limit = 1
            elif name == 'Detail':
                detObj = QTDetailPreferences()
                detObj.name = attrs.get('name', None)
                detObj.stylesheet = attrs.get('stylesheet', None)
                self._schema.details[detObj.name] = detObj
                # save current detail object for other use (recID)
                self._details = detObj
            elif name == 'RecordID':
                recIDObj = QTRecID(attrs.get('parametername', None), attrs.get('path', None))
                self._details.recID[recIDObj.parameter] = recIDObj
                    
                
    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def endElement(self, name):
        self._value = self._value.strip()
        if name == 'Schema':
            self._isSchema   = 0
            self.QTPrefObj.schemas[self._schema.label] = self._schema
            self.QTPrefObj.schemas[self._schema.NS] = self._schema
        if self._isSchema:
            if name == 'NotAvailableMessage':
                self._details.notAvailableMessage = self._value
        self._value = ''

    # _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
    def characters(self, ch):
        self._value += ch

        
def getPreferences(dsa, cfg):
    prefs = QTPreferences()
    prefHandler = QTPreferencesHandler(prefs)
    absFilename = os.path.join(cfg.datasourcesLocator, dsa, 'querytool_prefs.xml')
    xml.sax.parse(absFilename, prefHandler)
    return prefs
    