#
# -*- coding: utf-8 -*-

''' TBD '''


############################################################################################################
#
#   IMPORTS
#
#===========================================================================================================
from biocase.querytool import __version__
from biocase.tools.debugging import DebugClass
from biocase.tools.htmltools import escapeHtml
from biocase.wrapper.operations import recordStatus
from biocase.querytool.querygenerator import QueryGenerator
import libxml2
# xmlDOM.name : returns the node name
#      * type : returns a string indicating the node type
#      * content : returns the content of the node, it is based on xmlNodeGetContent() and hence is recursive.
#      * parent , children, last, next, prev, doc, properties: pointing to the associated element in the tree, those may return None in case no such link exists.

from mx.DateTime import now
import urllib





############################################################################################################
#
#   ERROR CLASSES
#
#===========================================================================================================
class WrapperError(Exception):
    def __init__(self):
        pass
        

############################################################################################################
#
#   CLASSES
#
#===========================================================================================================
class QueryDispatcher:
    '''A class to send queries to wrappers and do simple work with protocol responses.
    Supports DiGIR and BioCASe'''
    NS_DiGIR   = QueryGenerator.NS_DiGIR
    NS_BioCASe = QueryGenerator.NS_BioCASe
    NS_TAPIR   = QueryGenerator.NS_TAPIR
    
    # ------------------------------------------------------------------------------------
    def __init__(self, debug=DebugClass()):
        self._type          = None
        self.query          = None # the last query XML protocol string used
        self.response       = None # the response XML protocol string
        self.doc            = None # the response as a DOM object
        self.contentDOM     = None # the content DOM element of the response
        self.diagnosticsDOM = None # the diagnostics DOM element of the response
        self.recStatus      = recordStatus()
        self.debug          = debug
        
    # ------------------------------------------------------------------------------------
    def sendQuery(self, wrapper_url, protocolString, debuglevel=1, security_role=None):
        '''Sends a query to the wrapper and returns a record status object or None if query failed.'''
        self.query = protocolString
        # if digir destination parse resource of form RESOURCE@ACCESSPOINT
        if wrapper_url.count("@") == 1:
            digirResource, wrapper_url = wrapper_url.split("@")
            self.debug + "DiGIR resource %s, embedded in wrapper url, cut off." % (digirResource)
        self.debug + "SEND QUERY TO %s"%wrapper_url
        self.debug + "SECURITY ROLE=%s"%str(security_role)
        
        #We query with the result XML to the wrapper:
        # @request is used by DiGIR, @query by biocase
        if security_role is None:
            params = urllib.urlencode({'query': protocolString.encode('utf-8'), 'request': protocolString.encode('utf-8'),'debug':debuglevel})
        else:
            params = urllib.urlencode({'query': protocolString.encode('utf-8'), 'request': protocolString.encode('utf-8'), 'debug':debuglevel, 'role':security_role})
        #
        # risky. use error handling
        try:
            f = urllib.urlopen(wrapper_url, params)
            self.response = f.read()
            self.debug + "FULL WRAPPER RESPONSE %s"%unicode(self.response, 'utf-8')
        except:
            self.debug + "ERROR: WRAPPER URL %s COULD NOT BE OPENED" % wrapper_url
            return None
        # parse the xml into xmllib2 DOM:
        try:
            self.doc = libxml2.parseDoc(self.response)
        except:
            self.doc = None
            self.debug + "ERROR: NO WELL FORMED XML WRAPPER RESPONSE FOUND"
            return None
        self.debug + "ANALYZING RESPONSE DOC"
        root = self.doc.getRootElement()
        self.debug + "  DOC ROOT %s"%root.name
        if root.ns() is not None:
            try:
                self._type = unicode(root.ns()).replace('"','').replace("'",'')
                self._type = self._type[self._type.find('=')+1:]
                self.debug + "  PROTOCOL NS found: %s" % self._type
            except:
                self._type = None
        else:
            self.debug + "  PROTOCOL NS NOT found. ROOT ATTRIBUTES: %s" % root.get_properties()
        for header in root.children:
            if header.type == 'element' and header.name=='header':
                self.debug + "  Header section found."
                self.headerDOM = header
                break
        # try to get record status (paging info)
        elem = header
        while elem is not None:
            if elem.type == "element":
                if elem.name == 'content':
                    self.debug + "  Content section found."
                    self.contentDOM = elem
                # try to get diagnostics
                elif elem.name == 'diagnostics':
                    self.debug + "  Diagnostics section found."
                    self.diagnosticsDOM = elem
            elem = elem.next
        # DiGIR keeps record status in the diagnostics
        if self._type == QueryDispatcher.NS_DiGIR:
            for severity, code, diag in self.getDiagnostics():
                if code == 'MATCH_COUNT':
                    self.recStatus.total = int( diag )
                    self.debug + "    Record Total = %i"%(self.recStatus.total)
                elif code == 'RECORD_COUNT':
                    self.recStatus.count = int( diag )
                    self.debug + "    Record Count = %i"%(self.recStatus.count)
                elif code == 'END_OF_RECORDS':
                    pass
        # BioCASE
        elif self.contentDOM is not None:
            self.recStatus.start = int( self.contentDOM.prop('recordStart') )
            self.debug + "    Record Start = %i"%(self.recStatus.start)
            self.recStatus.count = int( self.contentDOM.prop('recordCount') )
            self.debug + "    Record Count = %i"%(self.recStatus.count)
            self.recStatus.drop = int( self.contentDOM.prop('recordDropped') )
            self.debug + "    Record Drop = %i"%(self.recStatus.drop)
            try:
                self.recStatus.total = int( self.contentDOM.prop('totalSearchHits') )
                self.debug + "    Record Total = %i"%(self.recStatus.total)
            except:
                self.recStatus.total = None                        
                self.debug + "    Record Total is not existing"
        #
        #    self.debug + "FATAL WRAPPER ERROR"
        #    raise WrapperError
        return self.recStatus
        
    # ------------------------------------------------------------------------------------
    def getScanValues(self):
        if self.contentDOM is None or self.contentDOM.children is None:
            return []
        result = []
        for val in self.contentDOM.children:
            # biocase
            if val.type == "element" and val.name == 'value':
                result.append( unicode(val.content, 'UTF-8' ) )
            # digir
            elif val.type == "element" and val.name == 'record':
                for n in val:
                    if n.type == "element":
                        result.append( unicode(n.content, 'UTF-8' ) )
                        break
        return result

    # ------------------------------------------------------------------------------------
    def getContent(self, asDoc=True):
        if self.contentDOM is None or self.contentDOM.children is None:
            self.debug + "COULDN'T GET CONTENT DOM NODE."
            return None
        contentDOM = self.contentDOM.copyNodeList()
        n = contentDOM.children
        while n is not None:
            nextnode = n.next
            if n.type == 'element':
                self.debug + "  Content payload element %s found." % n.name
            else:
                self.debug + "  Non payload element %s (type %s) ignored."%(n.name, n.type)
                n.freeNode()
                n.unlinkNode()
            n = nextnode
        # check how many payload elements content has...
        self.debug + "  ContentDOM.lsCountNode() = %i" % contentDOM.lsCountNode()
        if contentDOM.lsCountNode() < 1:
            self.debug + "NO CONTENT FOUND."
            return None
        elif contentDOM.lsCountNode() == 1:
            payload = contentDOM.children
        else:
            # more than one child element node of content. create artificial recordset node for this digir style
            payload = contentDOM
            payload.setName('RecordSet')
            payload.setNs( contentDOM.children.ns() )
        
        # create new doc with these nodes:
        if asDoc:
            doc = libxml2.newDoc("1.0")
            doc.setRootElement(payload)
        else:
            doc = payload
        self.debug + "CONTENT FOUND: %s" % unicode(doc)
        return doc

    # ------------------------------------------------------------------------------------
    def getDiagnostics(self):
        '''Return a list of triplet tuples of severity,code,content.'''
        diagnostics = []
        if self.diagnosticsDOM is not None and self.diagnosticsDOM.children is not None:
            for diag in self.diagnosticsDOM.children:
                if diag.type == "element" and diag.name == 'diagnostic':
                    diagnostics.append( unicode(diag.content, 'UTF-8' ) )
        return diagnostics



    